@@ -1,7 +1,7 @@
 # linuxmuster shell helperfunctions
 #
 # thomas@linuxmuster.net
-# 07.07.2015
+# 20170718
 # GPL v3
 #
 
@@ -166,7 +166,7 @@ removefrom_file() {
 validdomain() {
  [ -z "$1" ] && return 1
  tolower "$1"
-  if (expr match "$RET" '\([abcdefghijklmnopqrstuvwxyz0-9\-]\+\(\.[abcdefghijklmnopqrstuvwxyz0-9\-]\+\)\+$\)') &> /dev/null; then
+  if (expr match "$RET" '\([a-z0-9\-]\+\(\.[a-z0-9\-]\+\)\+$\)') &> /dev/null; then
     return 0
   else
     return 1
@@ -185,8 +185,7 @@ validip() {
 # test valid mac address syntax
 validmac() {
   [ -z "$1" ] && return 1
-  [ `expr length $1` -ne "17" ] && return 1
-  if (expr match "$1" '\([a-fA-F0-9-][a-fA-F0-9-]\+\(\:[a-fA-F0-9-][a-fA-F0-9-]\+\)\+$\)') &> /dev/null; then
+  if [ "$1" = "$(echo "$1" | sed -n "/^\([0-9a-fA-F][0-9a-fA-F]:\)\{5\}[0-9a-fA-F][0-9a-fA-F]$/p")" ]; then
     return 0
   else
     return 1
@@ -197,7 +196,7 @@ validmac() {
 validhostname() {
  [ -z "$1" ] && return 1
  tolower "$1"
- if (expr match "$RET" '\([abcdefghijklmnopqrstuvwxyz0-9\-]\+$\)') &> /dev/null; then
+ if (expr match "$RET" '\([a-z0-9\-]\+$\)') &> /dev/null; then
   return 0
  else
   return 1
@@ -390,7 +389,7 @@ test_maclist() {
    fi
   fi
  done
- 
+
  echo "$maclist_tested"
  return 0
 }
@@ -423,18 +422,18 @@ test_pwless_opsi(){
 
 # test if firewall can be connected passwordless
 test_pwless_fw(){
- if test_pwless_ssh "$ipcopip" 222 Firewall; then
+ if [ "$fwconfig" = "custom" ]; then
+  return 0
+ elif test_pwless_ssh "$ipcopip" 222 Firewall; then
   return 0
  else
   return 1
  fi
 }
 
-# returns ipfire, ipcop or none
+# returns ipfire, ipcop or custom
 get_fwtype(){
- local fwtype="custom"
- ssh -p 222 root@$ipcopip /bin/ls /var/ipfire &> /dev/null && fwtype="ipfire"
- echo "$fwtype"
+ echo "$fwconfig"
 }
 
 # check if urlfilter is active
@@ -451,6 +450,8 @@ check_urlfilter() {
 
 # execute a command on firewall
 exec_ipcop() {
+ #check type of firewall
+ [ "$fwconfig" = "custom" ] && return 0
  # test connection
  ssh -p 222 root@$ipcopip $* &> /dev/null || return 1
  return 0
@@ -458,17 +459,23 @@ exec_ipcop() {
 
 # execute a command on firewall width feedback
 exec_ipcop_fb() {
-  ssh -p 222 root@$ipcopip $*
+ #check type of firewall
+ [ "$fwconfig" = "custom" ] && return 0
+ ssh -p 222 root@$ipcopip $*
 }
 
 # fetch file from firewall
 get_ipcop() {
+ #check type of firewall
+ [ "$fwconfig" = "custom" ] && return 0
  scp -r -P 222 root@$ipcopip:$1 $2 &> /dev/null || return 1
  return 0
 }
 
 # upload file to firewall
 put_ipcop() {
+ #check type of firewall
+ [ "$fwconfig" = "custom" ] && return 0
  scp -r -P 222 $1 root@$ipcopip:$2 &> /dev/null || return 1
  return 0
 }
@@ -606,7 +613,7 @@ network_address_to_ips() {
    netmaskarr=(255 $((256-2**(16-${network[1]}))) 0 0)
   elif  [[ $((24-${network[1]})) > 0 ]]; then
    netmaskarr=(255 255 $((256-2**(24-${network[1]}))) 0)
-  elif [[ $((32-${network[1]})) > 0 ]]; then 
+  elif [[ $((32-${network[1]})) > 0 ]]; then
    netmaskarr=(255 255 255 $((256-2**(32-${network[1]}))))
   fi
  fi
@@ -634,7 +641,7 @@ discover_nics() {
 
  n=0
  # fetch all interfaces and their macs from /sys
- for i in /sys/class/net/bond* /sys/class/net/em* /sys/class/net/eth* /sys/class/net/br* /sys/class/net/wlan* /sys/class/net/intern /sys/class/net/extern /sys/class/net/dmz; do
+ for i in /sys/class/net/bond* /sys/class/net/eth* /sys/class/net/br* /sys/class/net/wlan* /sys/class/net/intern /sys/class/net/extern /sys/class/net/dmz; do
 
   [ -e $i/address ] || continue
 
@@ -745,6 +752,7 @@ assign_nics() {
 # uid=$1
 get_login_by_id() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select uid from userdata where id = '$1';"`
 }
 
@@ -752,6 +760,7 @@ get_login_by_id() {
 # username=$1
 get_uidnumber() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select uidnumber from posix_account where uid = '$1';"`
 }
 
@@ -759,6 +768,7 @@ get_uidnumber() {
 # group=$1
 get_gidnumber() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select gidnumber from groups where gid = '$1';"`
 }
 
@@ -767,7 +777,9 @@ get_gidnumber() {
 get_pgroup() {
   unset T_RET
   unset RET
+  [ -z "$1" ] && return 1
   T_RET=`psql -U ldap -d ldap -t -c "select gidnumber from posix_account where uid = '$1';"`
+  [ -z "$T_RET" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select gid from groups where gidnumber = '$T_RET';"`
 }
 
@@ -775,6 +787,7 @@ get_pgroup() {
 # username=$1
 get_homedir() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select homedirectory from posix_account where uid = '$1';"`
 }
 
@@ -782,6 +795,7 @@ get_homedir() {
 # username=$1
 get_realname() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select gecos from posix_account where uid = '$1';"`
 }
 
@@ -789,6 +803,7 @@ get_realname() {
 # group=$1
 get_pgroup_members() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select uid from memberdata where adminclass = '$1';"`
 }
 
@@ -796,6 +811,7 @@ get_pgroup_members() {
 # group=$1
 get_group_members() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select uid from memberdata where adminclass = '$1' or gid = '$1';"`
 }
 
@@ -803,6 +819,7 @@ get_group_members() {
 # group=$1
 check_project() {
   unset RET
+  [ -z "$1" ] && return 1
   RET=`psql -U ldap -d ldap -t -c "select gid from projectdata where gid = '$1';"`
   strip_spaces $RET
   [ "$RET" = "$1" ] && return 0
@@ -814,6 +831,8 @@ check_project() {
 check_group() {
   # check valid gid
   unset RET
+  [ -z "$1" ] && return 1
+
   get_gidnumber $1
   [ -z "$RET" ] && return 1
   [ "$RET" -lt 10000 ] && return 1
@@ -921,29 +940,17 @@ check_id() {
 # check if user is teacher
 # teacher=$1
 check_teacher() {
-  unset RET
   [ -z "$1" ] && return 1
-  local RC=1
-  get_group_members $TEACHERSGROUP
-  if echo "$RET" | grep -qw $1; then
-    RC=0
-  fi
-  unset RET
-  return $RC
+  groups "$1" | grep -qw "$TEACHERSGROUP" && return 0
+  return 1
 }
 
 # check if user is admin
 # admin=$1
 check_admin() {
-  unset RET
-  [ -z "$1" ] && return 1
-  local RC=1
-  get_group_members $DOMADMINS
-  if echo "$RET" | grep -qw $1; then
-    RC=0
-  fi
-  unset RET
-  return $RC
+ [ -z "$1" ] && return 1
+ groups "$1" | grep -qw "$DOMADMINS" && return 0
+ return 1
 }
 
 
@@ -995,3 +1002,88 @@ toupper() {
   [ -z "$1" ] && return 1
   RET=`echo $1 | tr a-z A-Z`
 }
+
+# converts a W.X.Y.Z IP address into its decimal representation
+function ip_to_dec
+{
+    local ip=$1
+    local octs=($(echo $ip | sed "s/\./ /g"))
+    local oct
+    local dec=0
+
+    for oct in ${octs[@]}; do
+        dec=$((($dec<<8)+($oct)))
+    done
+    echo $dec
+}
+
+# converts a decimal represented IP address into dotted notion W.X.Y.Z
+function dec_to_ip
+{
+    local dec=$1
+    local oct0
+    local oct1
+    local oct2
+    local oct3
+
+    oct0=$(($dec&255))
+    dec=$((dec>>8))
+    oct1=$(($dec&255))
+    dec=$((dec>>8))
+    oct2=$(($dec&255))
+    dec=$((dec>>8))
+    oct3=$(($dec&255))
+    dec=$((dec>>8))
+
+    echo "$oct3.$oct2.$oct1.$oct0"
+}
+
+# converts a bitwise netmask into dotted notion (e.g. /24 -> 255.255.255.0)
+function gen_netmask
+{
+    local l_ones=$1
+    local l_zeros=$((32-${l_ones}))
+    local ones=$(printf '1%.0s' $(seq -s " " ${l_ones}))
+    local zeros=$(printf '0%.0s' $(seq -s " " ${l_zeros}))
+    echo "ibase=2;${ones}${zeros}" | bc
+}
+
+# use global associative array global_subnets to store allowed subnets
+# it is up to the user of this function to initialize this data structure:
+#   unset global_subnets
+#   declare -A global_subnets
+function prepare_subnet_lookup_tab
+{
+    local networks=$1
+    local netaddr
+    local netaddr_dec
+    local netmask
+    local netmask_dec
+
+    for i in $networks; do
+        netaddr="$(echo $i | awk -F\/ '{ print $1}')"
+        netmask="$(echo $i | awk -F\/ '{ print $2}')"
+        netaddr_dec=$(ip_to_dec $netaddr)
+        netmask_dec=$(gen_netmask $netmask)
+        if  [ -z ${global_subnets[$netaddr_dec]} ] || [ $netmask_dec -lt ${global_subnets[$netaddr_dec]}]; then
+            global_subnets[$netaddr_dec]=$netmask_dec
+        fi
+    done
+}
+
+# uses global variable global_subnets in order to lookup whether a
+# supplied IP address is within given subnets. use function
+# prepare_subnet_lookup_tab in order to prepare lookup table
+function is_ip_in_subnets
+{
+    local ip=$1
+    local ip_dec=$(ip_to_dec $ip)
+
+    for netmask in ${global_subnets[@]}; do
+        netaddr=$(($ip_dec&$netmask))
+        if [ ! -z ${global_subnets[$netaddr]} ]; then
+            return 0
+        fi
+    done
+    return 1
+}
